#####################################################################################################################
# Quinn Heimann
# IPUMS-International
#
# This script calculates the most predominant/major ecoregion within each administrative unit.
# 
# Final output: Ecoregion.csv
#
# The script is set to re-run untill all units recieve valid values (values other than null).
#
# A few steps must be completed before and after processing.
# - specify files names and paths
# - inputs must be one shapefile (polygon) for ecoregion and one shapefile for units (polygon)
# - create the following folders inside processing folder: focals, tables, csv
# - delete any output that you won't need later. Do not delete output csv. Do not delete input raster
# and ecoregion shapefile, since you may need them for future processing.  
#   
######################################################################################################################

#import relevant libraries
import os, sys, string, arcpy, csv, shutil, datetime
from arcpy import env
from arcpy.sa import *

# Check out ArcGIS extensions
arcpy.CheckOutExtension("spatial")
arcpy.CheckOutExtension("GeoStats")

# have user input necessary parameters
working_directory = raw_input("Please specify the path to your working directory:\n")

ecoregion_filename = raw_input("Please specify the name of the ecoregion shapefile:\n") # specify path to the ecoregion shapefile
ecoregion_column_id = raw_input("Please specify the column containing your ecoregion identifying name or code:\n")

polygon_filename = raw_input("Please specify the name of the boundary shapefile:\n")
polygon_column_id = raw_input("Please specify the column containing your boundary identifying name or code:\n")

# Set start time
start = datetime.datetime.now()  
print 'start time %s\n\n' % (datetime.datetime.now()) 

# Environment settings
arcpy.env.workspace = working_directory
arcpy.env.overwriteOutput = True

# Paths to input data
ecoregion_path = working_directory + "\\" + ecoregion_filename
polygon_path = working_directory + "\\" + polygon_filename

# Paths to output data and directory creation
processing_folder = working_directory + "\\processing\\"
processing_tables_folder = processing_folder + "tables\\"
raster = processing_folder + "raster.tif"
out_extract = processing_folder + "extract_run.dbf"
out_clip = processing_folder + "clipped_ecoregion.shp"

if os.path.exists(processing_folder):
    shutil.rmtree(processing_folder)

os.mkdir(processing_folder)
os.mkdir(processing_tables_folder)

# Convert Polygon to Raster using the 5-digit ECO_ID
print "rasterizing the ecoregion polygon"
arcpy.PolygonToRaster_conversion(ecoregion_path, ecoregion_column_id, raster,"MAXIMUM_AREA", "", 0.0041665)
print 'finished polygon to raster conversion run.'
# Run Focal Statistics
print "Focal Statistics is in progress"
out_focal_stat = FocalStatistics(raster, "", "MAJORITY", "Data")
out_focal_stat.save(processing_folder + "focal_run" )
print 'finished focal run'

# Extract Values to Polygons
print "Extract Values is in progress"
ZonalStatisticsAsTable(polygon_path, polygon_column_id, processing_folder + "focal_run", out_extract, "DATA", "MAJORITY")
print 'finished values to Table run'

# Convert to table
arcpy.TableToGeodatabase_conversion(out_extract, processing_tables_folder)
print "Ecoregion Table is created."

arcpy.TableToTable_conversion(processing_tables_folder + "extract_run.dbf", working_directory, "ecoregion_results.csv")
print 'finished all run: %s\n\n' % (datetime.datetime.now() - start)

